# java.util.stream API for Hazelcast

hazelcast-jet-java.util.stream is a distributed implementation of the [java.util.stream API](https://docs.oracle.com/javase/8/docs/api/java/util/stream/package-summary.html)
 for Hazelcast, built using Hazelcast Jet, a distributed big data processing framework.

You can initialize a [Stream](https://docs.oracle.com/javase/8/docs/api/java/util/stream/Stream.html) instance from either a Hazelcast Map or List:

```java
HazelcastInstance instance = Hazelcast.newHazelcastInstance();
IMap<String, Integer> map = instance.getMap<String, Integer>("map");
IStreamMap<String, Integer> streamMap = IStreamMap.streamMap(map);
streamMap.stream()...
```
or

```java
IList<Integer> list = instance.getList("list");
IStreamList<Integer> streamList = IStreamList.streamList(list);
streamList.stream()...
```

## Examples

### Aggregation

```java
IMap<String, Integer> map = instance.getMap("salaries");
IStreamMap<Integer, String> streamMap = IStreamMap.streamMap(map);
long total = streamMap.stream().mapToLong(Map.Entry::getValue).sum();
```

### Word Count

```java
IMap<Integer, String> map = instance.getMap("words");
IStreamMap<Integer, String> streamMap = IStreamMap.streamMap(instance, map);
final Pattern space = Pattern.compile("\\s+");
Map<String, Long> wordCounts = streamMap.stream()
                .flatMap(m -> Stream.of(space.split(m.getValue())))
                .collect(DistributedCollectors.groupingBy(m -> m, DistributedCollectors.counting()));
```

## Additional Notes

- `DistributedCollectors` is a replacement for `Collectors` which includes serializable implementations of java.util.stream [Collectors](https://docs.oracle.com/javase/8/docs/api/java/util/stream/Collectors.html).
- Ordered streams execute sequentially on a single node, such as those originating from a IList. Use `unordered()` to make the stream unordered for parallel processing.
- All lambdas should be Serializable. As long as you are using the `Distributed.Stream<T>` interface, this will be automatically inferred by the compiler.
